from django_nose.tools import *
from assertions import BeExactlyAssertion as be_exactly, \
  BeValidAssertion as be_valid, EqualAssertion as equal, \
  EqualAssertion as be, HaveSelectorAssertion as have_selector, \
  HaveTextAssertion as have_text, RespondToAssertion as respond_to \

from base import RedRoverLiveTest, RedRoverTest, before, describe
from runner import RedRoverRunner


__all__ = [
  'RedRoverRunner', 'RedRoverLiveTest', 'RedRoverTest', 'before', 'describe',

  # RedRover assertions
  'be', 'be_exactly', 'be_valid', 'equal', 'have_selector', 'have_text',
  'respond_to',

  # Nose assertions
  'assert_true', 'assert_equal', 'assert_template_not_used',
  'assert_set_equal', 'assert_almost_equals', 'assert_greater_equal',
  'assert_sequence_equal', 'assert_not_equals', 'assert_is_none',
  'assert_raises', 'assert_almost_equal', 'assert_num_queries',
  'assert_not_in', 'assert_regexp_matches', 'assert_ok',
  'assert_not_almost_equals', 'assert_mail_count', 'assert_h_t_m_l_equal',
  'assert_is_instance', 'assert_not_regexp_matches', 'assert_contains',
  'assert_raises_regexp', 'assert_raises_message', 'assert_template_used',
  'assert_tuple_equal', 'assert_false', 'assert_field_output',
  'assert_not_contains', 'assert_less_equal', 'assert_list_equal',
  'assert_multi_line_equal', 'assert_items_equal', 'assert_form_error',
  'assert_equals', 'assert_not_is_instance', 'assert_is_not',
  'assert_h_t_m_l_not_equal', 'assert_dict_contains_subset', 'assert_greater',
  'assert_is', 'assert_redirects', 'assert_not_equal', 'assert_queryset_equal',
  'assert_dict_equal', 'assert_code', 'assert_in', 'assert_is_not_none',
  'assert_not_almost_equal', 'assert_less']



from urlparse import urlparse

from splinter.exceptions import ElementDoesNotExist


class ElementNotFound(ElementDoesNotExist):
  pass


def _splinter_action(_action, _parent):
  """
  All the various splinter actions we support.  (and a few others)

  """
  _browser = getattr(_parent, 'page')

  def visit(location):
    if location.startswith('http'):
      return _browser.visit(location)
    base_url = _parent.live_server_url
    return _browser.visit('%s%s' % (base_url, location))

  def click_link(locator):
    # First we'll try by text, then by CSS selector
    links = _browser.find_link_by_text(locator)
    if links:
      return links.first.click()
    links = _browser.find_by_css('a').find_by_css(locator)
    if links:
      return links.first.click()
    raise ElementNotFound()

  return locals().get(_action)


def get_splinter_actions(instance):
  """
  A dictionary of all supported Splinter actions plus a few of my own
  set up against the given test-class instance.

  """
  return {
    'click_link': _splinter_action('click_link', instance),
    'visit': _splinter_action('visit', instance),
    'current_path': _subject(
      instance.page, modifier=lambda url: urlparse(url).path)('url')}


def _subject(instance, parent_name=None, modifier=None):
  """
  The Subject object references a particular attribute and allows the
  developer to check it for various behaivors and conditions.

  """
  class Subject(object):

    def __init__(self, name):
      self.name = name
      self.instance = instance  # For debugging

    def should(self, cls, *args, **kwargs):
      assertion = cls(self.value, *args, **kwargs)
      if not assertion.passes:
        raise AssertionError(assertion.message)
      return True

    def should_not(self, cls, *args, **kwargs):
      assertion = cls(self.value, *args, **kwargs)
      if assertion.passes:
        raise AssertionError(assertion.message)
      return True

    @property
    def value(self):
      if parent_name:
        parent = getattr(instance, parent_name)
        _value = getattr(parent, self.name)
      else:
        _value = getattr(instance, self.name)
      if modifier:
        _value = modifier(_value)
      return _value

    def __str__(self):
      return str(self.value)

    def __repr__(self):
      return self.__str__()

    def __lt__(self, other):
      return self.value < other

    def __le__(self, other):
      return self.value <= other

    def __eq__(self, other):
      return self.value == other

    def __ne__(self, other):
      return self.value != other

    def __gt__(self, other):
      return self.value > other

    def __ge__(self, other):
      return self.value >= other

  return Subject
